﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using jabber.client;
using System.Threading;
using System.Diagnostics;
using JabberLoggingService.Properties;

namespace JabberLoggingService
{
    class JabberLoggingClient
    {
        // we will wait on this event until we're done sending
        static ManualResetEvent done = new ManualResetEvent(false);

        private System.Collections.Specialized.StringCollection TargetUsers;
        private string LogVault = "SYSTEM";

        public bool Shutdown = false;
        protected PerformanceCounter cpuCounter;


        public JabberLoggingClient(String Server, String User, String Password, System.Collections.Specialized.StringCollection _TargetUsers, String _LogVault, System.Collections.Specialized.StringCollection Filter)
        {
            JabberClient j = new JabberClient();
            // what user/pass to log in as
            j.User = User;
            bool takeit = true;
            j.Server = Server;
            j.Password = Password;


            TargetUsers = _TargetUsers;
            LogVault = _LogVault;

            // don't do extra stuff, please.
            j.AutoPresence = true;
            j.AutoRoster = true;
            j.AutoReconnect = 0;

            // listen for errors.  Always do this!
            j.OnError += new bedrock.ExceptionHandler(j_OnError);
            // what to do when login completes
            j.OnAuthenticate += new bedrock.ObjectHandler(j_OnAuthenticate);

            while (!Shutdown)
            {
                try
                {
                    // Set everything in motion
                    j.Connect();

                    // wait until sending a message is complete
                    done.WaitOne();

                    #region EventLog Loop
                    EventLog log = new EventLog();

                    log.Log = LogVault;

                    // get last log entry
                    EventLogEntry lastEntry = log.Entries[log.Entries.Count - 1];

                    EventLogEntry currentEntry;
                    EventLogEntry beforeEntry;
                    beforeEntry = lastEntry;
                    List<String> SendOut = new List<string>();
                    cpuCounter = new PerformanceCounter();

                    cpuCounter.CategoryName = "Processor";
                    cpuCounter.CounterName = "% Processor Time";
                    cpuCounter.InstanceName = "_Total";

                    done.WaitOne();

                    Int32 CPUCounter = 0;

                    float CPUAverage = cpuCounter.NextValue();

                    j.Presence(jabber.protocol.client.PresenceType.available, "CPU: " + Convert.ToInt32(cpuCounter.NextValue()) + "%", "Online", 0);

                    while (true)
                    {
                        Thread.Sleep(1000);

                        Settings.Default.Reload();

                        if (CPUCounter - 1 > 0)
                            CPUAverage = (CPUAverage + cpuCounter.NextValue()) / (CPUCounter - 1);

                        CPUCounter++;


                        if (CPUCounter == 11)
                        {
                            j.Presence(jabber.protocol.client.PresenceType.available, "CPU: " + Convert.ToInt32(CPUAverage) + "%", "Online", 0);
                            CPUCounter = 0;
                        }

                        SendOut.Clear();

                        if (log.Entries[log.Entries.Count - 1].TimeGenerated.Ticks != lastEntry.TimeGenerated.Ticks)
                        {
                            for (int i = log.Entries.Count - 1; i >= 0; i--)
                            {
                                currentEntry = log.Entries[i];

                                if (currentEntry.TimeGenerated.Ticks == lastEntry.TimeGenerated.Ticks)
                                {
                                    lastEntry = beforeEntry;
                                    break;
                                }
                                else
                                {
                                    SendOut.Insert(0, currentEntry.TimeGenerated.ToString() + " - " + currentEntry.Message);
                                }
                                beforeEntry = currentEntry;
                            }
                            if (SendOut.Count > 0)
                            {
                                foreach (String _str in SendOut)
                                {

                                    takeit = true;
                                    // check if we want to filter it
                                    foreach (String _Filter in Settings.Default.IgnoreKeywords)
                                    {
                                        if (_str.Contains(_Filter))
                                        {
                                            takeit = false;
                                            break;
                                        }
                                    }

                                    if (takeit)
                                    {
                                        foreach (String _User in Settings.Default.TargetUsers)
                                        {
                                            j.Message(_User, _str);
                                        }
                                    }
                                }
                            }
                        }
                    }
                    #endregion



                    // logout cleanly
                    j.Close();
                }
                catch (Exception)
                {

                }
                Thread.Sleep(100);
            }
        }

        private void j_OnAuthenticate(object sender)
        {
            try
            {
                // Sender is always the JabberClient.
                JabberClient j = (JabberClient)sender;

                foreach (String _User in TargetUsers)
                {
                    j.Message(_User, "Jabber Logging Service Version 0.2 started");
                }

                // Finished sending.  Shut down.
                done.Set();
            }
            catch (Exception)
            {
            }
        }

        static void j_OnError(object sender, Exception ex)
        {
            // There was an error!
            Console.WriteLine("Error: " + ex.ToString());

            // Shut down.
            done.Set();
        }
    }
}
